#! /usr/bin/mzscheme -qr

; configuration details
(define maxemailsize 65535) ; 64k chars
(define owner "Default Owner <markj@localhost>")
(define myaddr "Trubble Tracker <tracker@localhost>")
(define ticketdir "/tmp/trubble")
(define ticketfile (string-append ticketdir "/tickets.scm"))

(define replytext
"From: ~A
To: ~A
Subject: Re: [TN#~A] ~A

Thank you for your message.  I have used it to open a new problem report.
This has been forwarded to our support team.  If you reply to this
message, please include the part in square brackets on the subject line.
Doing so will mean that we can deal with your query more rapidly.
Thank you.

You will be sent a bcc copy of any replies to your message.

Your original message follows.  If you did not send this, please let us
know immediately:
\n~A")

; modules we use
(require
  (lib "string.ss")
  (lib "list.ss")
  (lib "process.ss")
  (lib "head.ss" "net"))

; library functions
(define add-to-file
  (lambda (number message)
    (let ((outp (open-output-file (format "~A/~A" ticketdir number) 'append)))
    (display message outp)
    (display "\n\n=================\n\n" outp))))

(define forward-to
  (lambda (emails message)
    (let ((ports (apply process* (cons "/usr/lib/sendmail" emails))))
      (display message (cadr ports))
      (close-output-port (cadr ports))
      (close-input-port (car ports))

(define reply-to
  (lambda (number email1 subject message)
    (let ((messout (format replytext myaddr email1 number subject message)))
      (forward-to (list email1) messout))))

(define get-reply-addr
  (lambda (message)
    (or
     (extract-field "Reply-To" message)
     (extract-field "From" message)
     (extract-field "Sender" message))))

; read in the ticket details
(let* ((tickets (read (open-input-file ticketfile)))
       (update! (lambda (updated)
                  (set! tickets
                    (map (lambda (x)
                           (if (= (car x) (car updated)) updated x))
                      tickets))))
       ; read email
       (email (read-string maxemailsize))
       (from (get-reply-addr email))
       ; find ticket number if present
       (tmat (regexp-match (regexp "\nSubject: .*\\[TN#([0-9]*) *(.*)\\]") email))
       (tnum (if tmat
                 (string->number (cadr tmat))
                 #f))
       (this (assoc tnum tickets)))
  ; if so, send it on
  (if (and tnum this)
      (begin
        (add-to-file tnum email)
        (forward-to (cons (list-ref this 3) (list-ref this 4)) email)
        ; execute any commands
        (let* ((commands (regexp-split (regexp " +") (caddr tmat)))
               (cmd1 (car commands)))
          (cond
                ((string=? cmd1 "notify")
                 (update!
                   (list tnum
                     (cadr this)
                     (caddr this)
                     (cadddr this)
                     (cons from (list-ref this 4)))))
                ((string=? cmd1 "adopt")
                  (update!
                   (list tnum
                     (cadr this)
                     (caddr this)
                     from
                     (list-ref this 4))))
                ((string=? cmd1 "close")
                 (update!
                   `(,tnum
                     (close ,@(cadr this))
                     ,@(list-tail this 2))))
                ((string=? cmd1 "flag")
                 (update!
                   `(,tnum
                     (,@(map string->symbol (cdr commands)) ,@(cadr this))
                     ,@(list-tail this 2))))
                ((string=? cmd1 "unflag")
                 (update!
                   `(,tnum
                     ,(remove* (map string->symbol (cdr commands)) (cadr this))
                     ,@(list-tail this 2)))))))
      
      ; if not, open a default ticket
      (let ((subj (cadr (regexp-match (regexp "\nSubject: ([^\n]*)") email)))
          (tnum (+ (apply max (map car tickets)) 1)))
        (set! tickets
          (cons
            `(,tnum
              (open)
              ,subj
              ,owner
              (,from))
            tickets))
        (add-to-file tnum email)
        (reply-to tnum from subj email)
        (forward-to (list owner) email)))
       
      ; write out the ticket details
      (write tickets (open-output-file ticketfile 'truncate/replace)))
